package guavacado;

import java.awt.Desktop;
import java.io.IOException;
import java.net.URI;
import java.util.List;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Hyperlink;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;

import com.sun.javafx.scene.control.skin.ScrollPaneSkin;

public class GuavHelp {
	private final static int ABOUT = 0;
	private final static int IMPORT = 1;
	private final static int EXPORT = 2;
	
	private Stage thisStage;
	
	@FXML
	private ScrollPane helpScrollPane;
	@FXML
	private VBox helpArea;
	@FXML
	private Button aboutButton;
	@FXML
	private Button importButton;
	@FXML
	private Button exportButton;
	
	private int helpIndex = -1;
	
	private Image importImage;
	private Image exportImage;
	private Image exportBatchImage;
	
	/**
	 * Prepares the help window.
	 * 
	 * @param thisStage The help window <code>Stage</code>.
	 * @param buttons Main Guavacado application buttons.
	 */
	protected void prepare(final Stage thisStage, List<Node> buttons) {
		this.thisStage = thisStage;
		helpScrollPane.setHmax(0);
		helpScrollPane.setHmin(0);
		
		importImage = ((ImageView)((Button)buttons.get(4)).getGraphic()).getImage();
		exportImage = ((ImageView)((Button)buttons.get(5)).getGraphic()).getImage();
		exportBatchImage = ((ImageView)((Button)buttons.get(6)).getGraphic()).getImage();
		
		thisStage.setOnShown(new EventHandler<WindowEvent>() {
			@Override
			public void handle(WindowEvent event) {
				ScrollPaneSkin skin = (ScrollPaneSkin)helpScrollPane.getSkin();
				for (Node node : skin.getChildren()) {
					if (node instanceof StackPane) {
						((StackPane)node).widthProperty().addListener(new ChangeListener<Number>() {
							@Override
							public void changed(ObservableValue <? extends Number> observable, Number oldVal, Number newVal) {
								helpArea.setMinWidth(newVal.doubleValue());
								helpArea.setMaxWidth(newVal.doubleValue());
								helpArea.setPrefWidth(newVal.doubleValue());
							}
						});
					}
				}
			}
		});
		
		exportButton.focusedProperty().addListener(new ChangeListener<Boolean>() {
			@Override
			public void changed(ObservableValue<? extends Boolean> observable, Boolean oldVal, Boolean newVal) {
				if (!newVal.booleanValue() && helpIndex == EXPORT) {
					exportButton.requestFocus();
				}
			}
		});
		importButton.focusedProperty().addListener(new ChangeListener<Boolean>() {
			@Override
			public void changed(ObservableValue<? extends Boolean> observable, Boolean oldVal, Boolean newVal) {
				if (!newVal.booleanValue() && helpIndex == IMPORT) {
					importButton.requestFocus();
				}
			}
		});
		aboutButton.focusedProperty().addListener(new ChangeListener<Boolean>() {
			@Override
			public void changed(ObservableValue<? extends Boolean> observable, Boolean oldVal, Boolean newVal) {
				if (!newVal.booleanValue() && helpIndex == ABOUT) {
					aboutButton.requestFocus();
				}
			}
		});
		
		populateAbout(null);
	}
	
	@FXML protected void dismiss(ActionEvent event) {
		thisStage.close();
	}
	
	@FXML protected void populateAbout(ActionEvent Event) {
		if (helpIndex != ABOUT) {
			helpIndex = ABOUT;
			helpScrollPane.setVvalue(0);
			aboutButton.requestFocus();
			
			helpArea.getChildren().clear();
			
			Label about = new Label("Guavacado");
			about.setStyle("-fx-text-fill: linear-gradient(from 0% 0% to 0% 100%, -fx-green, -fx-green 41%, derive(-fx-green, -40%) 62%, derive(-fx-green, -40%) 75%, -fx-green);"
					+ "-fx-font-size: 22px;"
					+ "-fx-font-weight: bold;");
			about.setWrapText(false);
			helpArea.getChildren().add(about);
			helpArea.setMargin(about, new Insets(0, 0, 13, 0));
			
			HBox versionBox = new HBox();
			versionBox.setSpacing(5);
			Label versionLabel = new Label("Version:");
			versionLabel.setStyle("-fx-text-fill: linear-gradient(from 0% 0% to 0% 100%, -fx-green, -fx-green 41%, derive(-fx-green, -40%) 62%, derive(-fx-green, -40%) 75%, -fx-green);"
					+ "-fx-font-size: 12px;"
					+ "-fx-font-weight: bold;");
			versionLabel.setWrapText(false);
			versionBox.getChildren().add(versionLabel);
			Label version = new Label(String.format("%.2f", GuavacadoMainWin.VERSION));
			version.setStyle("-fx-text-fill: rgb(107, 107, 107);"
					+ "-fx-font-size: 12px;");
			version.setWrapText(false);
			versionBox.getChildren().add(version);
			helpArea.getChildren().add(versionBox);
			
			HBox authorBox = new HBox();
			authorBox.setSpacing(7);
			Label authorLabel = new Label("Author:");
			authorLabel.setStyle("-fx-text-fill: linear-gradient(from 0% 0% to 0% 100%, -fx-green, -fx-green 41%, derive(-fx-green, -40%) 62%, derive(-fx-green, -40%) 75%, -fx-green);"
					+ "-fx-font-size: 12px;"
					+ "-fx-font-weight: bold;");
			authorLabel.setWrapText(false);
			authorBox.getChildren().add(authorLabel);
			Label author = new Label("Adam T. Ryder");
			author.setStyle("-fx-text-fill: rgb(107, 107, 107);"
					+ "-fx-font-size: 12px;");
			author.setWrapText(false);
			authorBox.getChildren().add(author);
			helpArea.getChildren().add(authorBox);
			
			HBox siteBox = new HBox();
			siteBox.setSpacing(20);
			Label siteLabel = new Label("Web:");
			siteLabel.setStyle("-fx-text-fill: linear-gradient(from 0% 0% to 0% 100%, -fx-green, -fx-green 41%, derive(-fx-green, -40%) 62%, derive(-fx-green, -40%) 75%, -fx-green);"
					+ "-fx-font-size: 12px;"
					+ "-fx-font-weight: bold;");
			siteLabel.setWrapText(false);
			siteBox.getChildren().add(siteLabel);
			Hyperlink site = new Hyperlink("http://1337atr.weebly.com");
			site.setWrapText(false);
			siteBox.getChildren().add(site);
			helpArea.getChildren().add(siteBox);
			
			site.setOnAction(new EventHandler<ActionEvent>() {
				@Override
				public void handle(ActionEvent event) {
					try {
						String os = System.getProperty("os.name").toLowerCase();
						if (Desktop.getDesktop().isSupported(Desktop.Action.BROWSE)) {
							try {
								Desktop.getDesktop().browse(URI.create("http://1337atr.weebly.com"));
							} catch (IOException e) {
								
							}
						} else {
							try {
								if (os.indexOf("win") >= 0) {
									Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + URI.create("http://1337atr.weebly.com"));
								} else if (os.indexOf("mac") >= 0) {
									Runtime.getRuntime().exec("open " + URI.create("http://1337atr.weebly.com"));
								} else if (os.indexOf("nix") >=0 || os.indexOf("nux") >= 0) {
									Runtime.getRuntime().exec("xdg-open " + URI.create("http://1337atr.weebly.com"));
								}
							} catch (IOException e) {
								
							}
						}
					} catch (UnsupportedOperationException unE) {
						
					}
				}
			});
		}
	}
	
	@FXML protected void populateImportEdit(ActionEvent event) {
		if (helpIndex != IMPORT) {
			helpIndex = IMPORT;
			helpScrollPane.setVvalue(0);
			importButton.requestFocus();
			
			helpArea.getChildren().clear();
			
			helpArea.getChildren().add(new ImageView(importImage));
			Label l1 = new Label("Import images to your project by clicking the import "
					+ "button pictured above.  Upon clicking the import button you will be "
					+ "presented with a file browser window, navigate to the images you'd "
					+ "like to import, select one or more and then click the open button.  "
					+ "You should see one or more image tiles slide into the main project "
					+ "view area.  Guavacado will display a progress indicator in place of "
					+ "the image thumbnail on each image tile until the image has been loaded.");
			l1.setWrapText(true);
			helpArea.getChildren().add(l1);
			
			HBox closeBox = new HBox();
			closeBox.setSpacing(12);
			closeBox.setAlignment(Pos.CENTER_LEFT);
			closeBox.getChildren().add(new ImageView(IconTile.closeTileIcon));
			Label l2 = new Label("Remove an image from your project by clicking the close button.");
			l2.setWrapText(true);
			closeBox.getChildren().add(l2);
			helpArea.getChildren().add(closeBox);
			helpArea.setMargin(closeBox, new Insets(15, 0, 7, 0));
			
			HBox duplicateBox = new HBox();
			duplicateBox.setSpacing(12);
			duplicateBox.setAlignment(Pos.CENTER_LEFT);
			duplicateBox.getChildren().add(new ImageView(IconTile.duplicateTileIcon));
			Label l3 = new Label("Duplicate an image in your project by clicking the duplicate button.");
			l3.setWrapText(true);
			duplicateBox.getChildren().add(l3);
			helpArea.getChildren().add(duplicateBox);
			helpArea.setMargin(duplicateBox, new Insets(0, 0, 20, 0));
			
			helpArea.getChildren().add(new ImageView(new Image("/guavacado/resources/gfx/TextArea.png")));
			Label l4 = new Label("You can modify the width and height resolution of the image "
					+ "by clicking in one of the left most text boxes, pictured above, within the image tile.  "
					+ "The top box refers to the image width while the box just below that is "
					+ "used to modify the image height.  Leaving one or more of these boxes "
					+ "blank will indicate that the width, height or both should not be resized.");
			l4.setWrapText(true);
			helpArea.getChildren().add(l4);
			helpArea.setMargin(l4, new Insets(5, 0, 20, 0));
			
			HBox avacadoBox = new HBox();
			avacadoBox.setSpacing(7);
			avacadoBox.getChildren().addAll(new ImageView(GuavacadoMainWin.avacadoToggleLockedSmall), new ImageView(GuavacadoMainWin.avacadoToggleUnlockedSmall));
			helpArea.getChildren().add(avacadoBox);
			
			Label l5 = new Label("To the immediate left of the scale boxes you should see an "
					+ "avacado.  Clicking the avacado to reveal a sliced open avacado (above "
					+ "right) will indicate that the width/height scales can be edited "
					+ "independently of one another while a closed, whole, avacado (above left) "
					+ "indicates that the two are aspect ratio dependent therefore editing one"
					+ " will change the other so that the width/height ratio of the output "
					+ "image will match the input image.");
			l5.setWrapText(true);
			helpArea.getChildren().add(l5);
			helpArea.setMargin(l5, new Insets(5, 0, 20, 0));
			
			helpArea.getChildren().add(new ImageView(new Image("/guavacado/resources/gfx/BitDepthBox.png")));
			Label l6 = new Label("Use the bit depth drop down box to modify the bit depth of "
					+ "your output image.  Not all bit depths are supported by all of "
					+ "Guavacados supported formats, see the Export section for more "
					+ "information.");
			l6.setWrapText(true);
			helpArea.getChildren().add(l6);
			helpArea.setMargin(l6, new Insets(5, 0, 20, 0));
			
			helpArea.getChildren().add(new ImageView(new Image("/guavacado/resources/gfx/CheckBox.png")));
			Label l7 = new Label("The compression box is used only for .ico export.  When "
					+ "selected the output image will use PNG compression when written to the "
					+ "icon file.  Icons with PNG compression are only supported in Windows "
					+ "Vista and later.");
			l7.setWrapText(true);
			helpArea.getChildren().add(l7);
			helpArea.setMargin(l7, new Insets(5, 0, 20, 0));
		}
	}
	
	@FXML protected void populateExport(ActionEvent event) {
		if (helpIndex != EXPORT) {
			helpIndex = EXPORT;
			helpScrollPane.setVvalue(0);
			exportButton.requestFocus();
			
			helpArea.getChildren().clear();
			
			helpArea.getChildren().add(new ImageView(exportBatchImage));
			Label l1 = new Label("To export your project as seperate images in the PNG, JPG or "
					+ "BMP format select the batch export button pictured above.  You will be "
					+ "presented with a directory chooser window, navigate to the directory you "
					+ "wish to save the images in and click Select Folder.  At this point you "
					+ "will be asked to choose the output format, either PNG, JPG or BMP and, "
					+ "if applicable, the quality of the resulting image.");
			l1.setWrapText(true);
			helpArea.getChildren().add(l1);
			helpArea.setMargin(l1, new Insets(5, 0, 20, 0));
			
			Label l2 = new Label("The PNG file format does not support 16-bit depths, "
					+ "exporting an image to PNG with the 16-bit option selected will result "
					+ "in an image that has been reduced to 16-bit color depth, but written to "
					+ "disk as 24-bit.  When selecting JPG output the output image will be  "
					+ "converted to the selected bit-depth, but will be written to disk as "
					+ "24-bit with the exception of 32-bit, selecting 32-bit output with JPG "
					+ "compression will result in an image converted to and written as 24-bit.  "
					+ "BMP will match all bit-depths with the exception of 32-bit, any image "
					+ "selected with 32-bit depth will be converted to 24-bit.");
			l2.setWrapText(true);
			helpArea.getChildren().add(l2);
			helpArea.setMargin(l2, new Insets(0, 0, 20, 0));
			
			helpArea.getChildren().add(new ImageView(exportImage));
			Label l3 = new Label("Click the export project as Icon, .ico, button pictured above"
					+ " to export all of the images in your project to a single icon file.  "
					+ "Icon files do not support 16-bit images, any image with selected 16-bit "
					+ "depth will be converted to and written as 24-bit.");
			l3.setWrapText(true);
			helpArea.getChildren().add(l3);
			helpArea.setMargin(l3, new Insets(5, 0, 20, 0));
		}
	}
}
