package guavacado;

import java.util.ArrayList;
import java.util.List;

import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;

/**
 * Guavacado is an application for batch resizing/exporting images to the
 * ico, png, jpg or bmp format.
 * 
 * Feel free to distribute, modify and use this code in your own projects
 * without restriction.
 * 
 * @author Adam T. Ryder http://1337atr.weebly.com
 * @version 1.0
 *
 */
public class Main extends Application {
	
	private static String[] arguments;

	public static void main(String[] args) {
		arguments = args;
		launch(args);
	}
	
	@Override
	public void start(Stage primaryStage) {
		List<String> args = new ArrayList<String>();
		for (String arg : arguments) {
			if (isSupported(arg)) {
				args.add(arg);
			}
		}
		arguments = null;
		
		try {
			FXMLLoader loader = new FXMLLoader(Main.class.getResource("/guavacado/resources/fxml/GuavaMain.fxml"));
			Pane mainPane = (Pane) loader.load();
			((GuavacadoMainWin) loader.getController()).prepare(primaryStage, args);
			
			Scene scene = new Scene(mainPane);
			scene.getStylesheets().add(Main.class.getResource("/guavacado/resources/fxml/GuavacadoTheme.css").toExternalForm());
			primaryStage.setScene(scene);
			primaryStage.getIcons().addAll(new Image("/guavacado/resources/gfx/GuavacadoIcon_72_16_32bit.png"), new Image("/guavacado/resources/gfx/GuavacadoIcon_72_24_32bit.png"), new Image("/guavacado/resources/gfx/GuavacadoIcon_72_32_32bit.png"), new Image("/guavacado/resources/gfx/GuavacadoIcon_72_48_32bit.png"), new Image("/guavacado/resources/gfx/GuavacadoIcon_72_64_32bit.png"), new Image("/guavacado/resources/gfx/GuavacadoIcon_72_72_32bit.png"));
			primaryStage.setTitle("Guavacado");
			primaryStage.show();
			double tmpW = scene.getWidth();
			double tmpH = scene.getHeight();
			double tmpX = primaryStage.getX();
			double tmpY = primaryStage.getY();
			primaryStage.setResizable(false);
			if (tmpW != scene.getWidth()) {
				primaryStage.setWidth(primaryStage.getWidth() - (scene.getWidth() - tmpW));
				primaryStage.setX(tmpX);
			}
			if (tmpH != scene.getHeight()) {
				primaryStage.setHeight(primaryStage.getHeight() - (scene.getHeight() - tmpH));
				primaryStage.setY(tmpY);
			}
		} catch (Exception e) {
			System.out.println("Unable to start Guavacado due to the following error:");
			System.out.println(e.getMessage());
			System.exit(1);
		}
	}
	
	/**
	 * Checks to see if a file is a Guavacado Project File.
	 * 
	 * @param file Path to the file in question.
	 * @return True if the file is a Guavacado Project File.
	 */
	public static boolean isGPF(final String file) {
		int index = file.lastIndexOf(".");
		if (index >= 1) {
			if (file.substring(index + 1).toLowerCase().equals("gpf")) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Check to see if a file is supported by Guavacado.
	 * 
	 * @param file Path to the file in question.
	 * @return True if the file is supported otherwise false.
	 */
	public static boolean isSupported(final String file) {
		int index = file.lastIndexOf(".");
		if (index >= 1) {
			String ext = file.substring(index + 1).toLowerCase();
			switch(ext) {
			case "png":
				return true;
			case "jpg":
				return true;
			case "jpeg":
				return true;
			case "bmp":
				return true;
			case "gpf":
				return true;
			}
		}
		
		return false;
	}

}
